import { Module } from '@nestjs/common';
import { APP_FILTER, APP_INTERCEPTOR, APP_GUARD } from '@nestjs/core';
import { TypeOrmModule } from '@nestjs/typeorm';
import { AppController } from './app.controller';
import { AppService } from './app.service';
import { Connection } from 'typeorm';
import { UserModule } from './user/user.module';

import { HttpErrorFilter } from './shared/http-error.filter';
import { LoggingInterceptor } from './shared/logging.interceptor';
import { NotificationModule } from './notifications/notifications.module';

import { GameModule } from './game/game.module';
import { RolesGuard } from './shared/roles.guard';
import { TaskModule } from './task/task.module';
import { DrawModule } from './draw/draw.module';

@Module({
  imports: [
    TypeOrmModule.forRoot(),
    UserModule,
    GameModule,
    NotificationModule,
    TaskModule,
    DrawModule,
  ],
  controllers: [AppController],
  providers: [
    AppService,
    {
      provide: APP_FILTER,
      useClass: HttpErrorFilter,
    },
    {
      provide: APP_INTERCEPTOR,
      useClass: LoggingInterceptor,
    },
    {
      provide: APP_GUARD,
      useClass: RolesGuard,
    },
  ],
})
export class AppModule {
  constructor(private readonly connection: Connection) {}
}
