import {
  SubscribeMessage,
  WebSocketGateway,
  WebSocketServer,
  OnGatewayInit,
  OnGatewayConnection,
  OnGatewayDisconnect,
} from '@nestjs/websockets';
import { Logger } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Server, Socket } from 'socket.io';
import { Repository } from 'typeorm';

import { NotificationEntity } from './notification.entity';

@WebSocketGateway()
export class NotificationGateway
  implements OnGatewayInit, OnGatewayConnection, OnGatewayDisconnect {
  constructor(
    //create references to tables as repositories
    @InjectRepository(NotificationEntity)
    private notificationRepository: Repository<NotificationEntity>,
  ) {}
  @WebSocketServer()
  server: Server;

  // for debugging: starting server, accepting connection, terminating connection
  private logger: Logger = new Logger('NotificationGateway');

  afterInit(server: Server) {
    this.logger.log('Server has started');
  }

  handleConnection(client: Socket, ...args: any[]) {
    this.logger.log(`Client ${client.id} connected`);
  }

  handleDisconnect(client: Socket) {
    this.logger.log(`Client ${client.id} disconnected`);
  }

  // notifications for when game needs to be paused / stopped
  @SubscribeMessage('shutItDown')
  async handleMessage(client: Socket, text: string) {
    this.logger.log(text);
    // send the message to all clients listening to warningToPlayers branch
    this.server.emit('warningToPlayers', text);
    // create entity properties
    const message = await this.notificationRepository.create({ message: text });
    // insert created entity NOTE: insert method doesn't check for duplicates.
    await this.notificationRepository.insert(message);
  }
}
